#include <stdlib.h>
#include <stdio.h>
#include <ctype.h>
#include "inout.h"	// header file generated by MIDL compiler



void Usage( char *pszProgramName )
{
	fprintf( stderr, "Usage:  %s\n", pszProgramName );
	fprintf( stderr, " -a {0..6}      authentication off/level .................. [0]\n" );
	fprintf( stderr, "    0=off 1=default 2=none 3=connect 4=call 5=pkt 6=privacy\n" );
	fprintf( stderr, " -s {0|1}       use security callback (no/yes) ............ [0]\n" );
	fprintf( stderr, " -d princname   principal name ............................ [midlbug]\n" );
	fprintf( stderr, " -p protseq     protseq to use ............................ [ncalrpc]\n" );
	fprintf( stderr, " -e endpoint    endpoint to use ........................... [foo]\n" );
	fprintf( stderr, " -m maxcalls    max # calls active ........................ [20]\n" );
	fprintf( stderr, " -n mincalls    min # call buffers ........................ [1]\n" );
	fprintf( stderr, " -f {0|1}       wait/ret from listen ...................... [0]\n" );
	exit( 1 );
}



int useauthn = 0; // global authentication enable/disable
int use_security_callback; // do we kick off unauth'ed users with a callback, or
	// by checking in every server call?



RPC_STATUS __stdcall security_callback( RPC_IF_ID *iface, void *ctx )
{
	RPC_STATUS rc, i;
	unsigned long level, authnsvc, authzsvc;
	unsigned char *princname = NULL, *uuid = NULL;
	RPC_AUTHZ_HANDLE foo = NULL;

	UuidToString( &iface->Uuid, &uuid );
	printf( "callback: call to interface %s, v%d.%d\ncallback: binding/context at %ph\n",
		uuid, (int) iface->VersMajor, (int) iface->VersMinor, ctx );
	if ( uuid != NULL )
		RpcStringFree( &uuid );

	rc = RpcBindingInqAuthClient( ctx, &foo, &princname, &level, &authnsvc, &authzsvc );

	if ( rc )
	{
		printf( "callback: RpcBindingInqAuthClient() returned %d, denying access\n", rc );
		return 69;
	}

	if ( princname != NULL )
		RpcStringFree( &princname );

	i = RpcImpersonateClient( NULL );

	if ( i != 0 )
	{
		printf( "callback: RIC() returned %d, denying access\n", i );
		return 70;
	}
	else
	{
		char uname[200];
		DWORD j;
		j = sizeof uname;
		if ( GetUserName( uname, &j ) )
			printf( "callback: user name \"%s\", allowed to access our interface\n", uname );
		RpcRevertToSelf();
	}

	// user probably is OK
	return RPC_S_OK;
}



int _CRTAPI1 main( int argc, char *argv[] )
{
	RPC_STATUS status;
	unsigned char * pszProtocolSequence = "ncalrpc";
	SECURITY_DESCRIPTOR sd;
	unsigned char * pszEndpoint	= "foo";
	char *defprincname = "midlbug";
	unsigned int    cMinCalls = 1;
	unsigned int    cMaxCalls = 20;
	unsigned int    fDontWait = FALSE;
	int i;

	/* allow the user to override settings with command line switches */
	for ( i = 1; i < argc; i++ )
	{
		if ( *argv[i] == '-' || *argv[i] == '/' )
		{
			switch ( tolower( argv[i][1] ) )
			{
			case 'p':  // protocol sequence
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				pszProtocolSequence = argv[++i];
				break;
			case 'e':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				pszEndpoint = argv[++i];
				break;
			case 'm':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				cMaxCalls = (unsigned int) atoi( argv[++i] );
				break;
			case 'n':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				cMinCalls = (unsigned int) atoi( argv[++i] );
				break;
			case 'a':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				useauthn = atoi( argv[++i] );
				break;
			case 'd':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				defprincname = argv[++i];
				break;
			case 'f':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				fDontWait = (unsigned int) atoi( argv[++i] );
				break;
			case 's':
				if ( i + 1 > argc - 1 )
				{
					fprintf( stderr, "switch \"%s\" requires an argument.\n", argv[i] );
					Usage( argv[0] );
				}
				use_security_callback = atoi( argv[++i] );
				break;
			case 'h':
			case '?':
			default:
				Usage( argv[0] );
			}
		}
		else
			Usage( argv[0] );
	}

	if ( useauthn )
	{
		// ncalrpc and ncacn_np require a security descriptor -- both derive info from
		// NT's security subsystem by creating some NT object symbolizing the endpoint,
		// and both secure the endpoint with an NT SD to protect it.  Here, I grant access
		// to whoever ambles along, on the assumption that manager functions will examine
		// client credentials, privileges, and rights.

		if ( !InitializeSecurityDescriptor( &sd, SECURITY_DESCRIPTOR_REVISION ) )
			printf( "InitSD() failed, error = %d\n", GetLastError() );

		if ( !SetSecurityDescriptorDacl( &sd, TRUE, NULL, FALSE ) )
			printf( "SSDD() failed, error = %d\n", GetLastError() );
	}

	status = RpcServerUseProtseqEp( pszProtocolSequence, cMaxCalls, pszEndpoint,
		useauthn? &sd: NULL );
	printf( "RpcServerUseProtseqEp() returned %d\n", status );
	if ( status )
		exit( status );

	if ( useauthn )
	{
		char *princname = NULL;
		OSVERSIONINFO osvi;

		memset( &osvi, '\0', sizeof osvi );
		osvi.dwOSVersionInfoSize = sizeof osvi;
		GetVersionEx( &osvi );

		if ( osvi.dwPlatformId == VER_PLATFORM_WIN32_WINDOWS ) // Win95?
		{
			status = RpcServerInqDefaultPrincName( RPC_C_AUTHN_WINNT, &princname );
			printf( "Win95 RpcServerInqDefaultPrincName() returned %d {%s}\n",
			  status, princname == NULL? "*NULL*": princname );
			if ( princname == NULL )
			{
				princname = defprincname;
				printf( "Win95, but still using principal name {%s}\n", princname );
			}
		}
		else
		{
			princname = defprincname;
			printf( "Not Win95, using principal name {%s}\n", princname );
		}

		status = RpcServerRegisterAuthInfo( princname, RPC_C_AUTHN_WINNT, NULL, NULL );
		printf( "RpcServerRegisterAuthInfo() returned %d\n", status );
		if ( status )
			exit( status );
	}

	if ( useauthn && use_security_callback )
	{
		// note: RPC_C_PROTSEQ_MAX_REQS_DEFAULT is ignored,
		// as we don't allow autolisten on our interface.
		printf( "our interface pointer is %p\n", inout_v1_0_s_ifspec );
		status = RpcServerRegisterIfEx( inout_v1_0_s_ifspec, NULL, NULL,
			0, RPC_C_PROTSEQ_MAX_REQS_DEFAULT, security_callback );
	}
	// no security callback, register as usual
	else
		status = RpcServerRegisterIf( inout_v1_0_s_ifspec, NULL, NULL );

	printf( "RpcServerRegisterIf[Ex]() returned %d\n", status );
	if ( status )
		exit( status );

	printf( "Calling RpcServerListen\n" );
	status = RpcServerListen( cMinCalls, cMaxCalls, fDontWait );
	printf( "RpcServerListen() returned %d\n", status );
	if ( status )
		exit( status );

	if ( fDontWait )
	{
		printf( "Calling RpcMgmtWaitServerListen\n" );
		status = RpcMgmtWaitServerListen();  //  wait operation
		printf( "RpcMgmtWaitServerListen() returned %d\n", status );
		if ( status )
			exit( status );
	}

	return 0;
}



/*********************************************************************/
/*				 MIDL allocate and free							*/
/*********************************************************************/



void __RPC_FAR * __RPC_USER midl_user_allocate( size_t len )
{
	return malloc( len );
}



void __RPC_USER midl_user_free( void __RPC_FAR *ptr )
{
	free( ptr );
}

/* end file inouts.c */

